unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,DirectxGraphics,DXGUtils,
  AppEvnts;

type
  TD3DFrm = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    D3D:iDirect3D8;//  
    D3DD:iDirect3DDevice8;//  
    VB:iDirect3DVertexBuffer8;//  
    function InitD3D:HResult;// Direct3D
    function InitVertexBuffer:HResult;//    
    function RenderScene:HResult;//
    procedure EnableLighting;// 
    procedure CleanUp;// 
  public
    { Public declarations }
  end;

// 
type
  TCUSTOMVERTEX = packed record
    X,Y,Z:Single;//  
    nX,nY,nZ : Single;// 
  end;

const
  D3DFVF_CUSTOMVERTEX=D3DFVF_XYZ or D3DFVF_NORMAL;//    

var
  D3DFrm: TD3DFrm;
  Verts:array[0..47] of TCUSTOMVERTEX;//    
  RedM,GreenM:TD3DMaterial8;//   
  Angle:Single;// 

implementation

{$R *.DFM}

procedure TD3DFrm.FormCreate(Sender: TObject);
var
   HRet:HResult;
begin
   Randomize;
   // Direct3D
   HRet:=InitD3D;
   if FAILED(HRet) then
   begin
      CleanUp;
      MessageBox(handle,PChar(DXGErrorString (HRet)),'InitD3D',0);
   end;
   //    
   HRet :=InitVertexBuffer;
   if FAILED(HRet) then
   begin
      CleanUp;
      MessageBox(handle,PChar(DXGErrorString (HRet)),'InitVertexBuffer',0);
   end;
   //    
   EnableLighting;
   Angle:=0;
end;

procedure TD3DFrm.FormDestroy(Sender: TObject);
begin
   CleanUp;
end;

procedure TD3DFrm.CleanUp;
begin
   if Assigned(VB) then
   begin
      VB._Release;
      VB:=nil;
   end;
   if Assigned(D3DD) then
   begin
      D3DD._Release;
      D3DD:=nil;
   end;
   if Assigned(D3D) then
   begin
      D3D._Release;
      D3D:=nil;
   end;
end;

function TD3DFrm.InitD3D;
var
   D3DDM:TD3DDisplayMode;
   D3DPP:TD3DPresent_Parameters;
   HRet:HResult;
begin
   D3D:=Direct3DCreate8(D3D_SDK_Version);//   

   //    
   HRet:=D3D.GetAdapterDisplayMode(D3DAdapter_Default,D3DDM);
   if FAILED(HRet) then
   begin
      CleanUp;
      MessageBox(handle,PChar(DXGErrorString (HRet)),'GetAdapterDisplayMode',0);
   end;

   ZeroMemory(@D3DPP,SizeOf(D3DPP));// 
   With D3DPP do
   begin
      Windowed:=True;//  
      SwapEffect:=D3DSWAPEFFECT_DISCARD;//  
      BackBufferFormat:=D3DDM.Format;//  
      EnableAutoDepthStencil:=True;//  Z-
      AutoDepthStencilFormat:=D3DFMT_D16;// Z-
   end;

   Result:=D3D.CreateDevice(D3DAdapter_Default,// 
                           D3DDEVTYPE_HAL,// 3D Accelerator
                           Handle,// 
                           D3DCREATE_SOFTWARE_VERTEXPROCESSING,//  CPU
                           D3DPP,// 
                           D3DD);// 
end;

function TD3DFrm.InitVertexBuffer;
var
   HRet:HResult;
   PVerts:PByte;
begin
   //  

   //
   Verts[0].X:=2;
   Verts[0].Y:=0;
   Verts[0].Z:=-4;
   Verts[0].NX:=0;
   Verts[0].NY:=-1;
   Verts[0].NZ:=0;

   Verts[1].X:=2;
   Verts[1].Y:=0;
   Verts[1].Z:=4;
   Verts[1].NX:=0;
   Verts[1].NY:=-1;
   Verts[1].NZ:=0;

   Verts[2].X:=-2;
   Verts[2].Y:=0;
   Verts[2].Z:=4;
   Verts[2].NX:=0;
   Verts[2].NY:=-1;
   Verts[2].NZ:=0;
   //
   Verts[3].X:=-2;
   Verts[3].Y:=0;
   Verts[3].Z:=4;
   Verts[3].NX:=0;
   Verts[3].NY:=-1;
   Verts[3].NZ:=0;

   Verts[4].X:=-2;
   Verts[4].Y:=0;
   Verts[4].Z:=-4;
   Verts[4].NX:=0;
   Verts[4].NY:=-1;
   Verts[4].NZ:=0;

   Verts[5].X:=2;
   Verts[5].Y:=0;
   Verts[5].Z:=-4;
   Verts[5].NX:=0;
   Verts[5].NY:=-1;
   Verts[5].NZ:=0;
   //
   Verts[6].X:=2;
   Verts[6].Y:=0;
   Verts[6].Z:=4;
   Verts[6].NX:=1;
   Verts[6].NY:=0;
   Verts[6].NZ:=0;

   Verts[7].X:=2;
   Verts[7].Y:=0;
   Verts[7].Z:=-4;
   Verts[7].NX:=1;
   Verts[7].NY:=0;
   Verts[7].NZ:=0;

   Verts[8].X:=2;
   Verts[8].Y:=3;
   Verts[8].Z:=4;
   Verts[8].NX:=1;
   Verts[8].NY:=0;
   Verts[8].NZ:=0;
   //
   Verts[9].X:=2;
   Verts[9].Y:=0;
   Verts[9].Z:=-4;
   Verts[9].NX:=1;
   Verts[9].NY:=0;
   Verts[9].NZ:=0;

   Verts[10].X:=2;
   Verts[10].Y:=3;
   Verts[10].Z:=-4;
   Verts[10].NX:=1;
   Verts[10].NY:=0;
   Verts[10].NZ:=0;

   Verts[11].X:=2;
   Verts[11].Y:=3;
   Verts[11].Z:=4;
   Verts[11].NX:=1;
   Verts[11].NY:=0;
   Verts[11].NZ:=0;
   //
   Verts[12].X:=-2;
   Verts[12].Y:=0;
   Verts[12].Z:=4;
   Verts[12].NX:=0;
   Verts[12].NY:=0;
   Verts[12].NZ:=1;

   Verts[13].X:=2;
   Verts[13].Y:=0;
   Verts[13].Z:=4;
   Verts[13].NX:=0;
   Verts[13].NY:=0;
   Verts[13].NZ:=1;

   Verts[14].X:=2;
   Verts[14].Y:=3;
   Verts[14].Z:=4;
   Verts[14].NX:=0;
   Verts[14].NY:=0;
   Verts[14].NZ:=1;
   //
   Verts[15].X:=-2;
   Verts[15].Y:=0;
   Verts[15].Z:=4;
   Verts[15].NX:=0;
   Verts[15].NY:=0;
   Verts[15].NZ:=1;

   Verts[16].X:=2;
   Verts[16].Y:=3;
   Verts[16].Z:=4;
   Verts[16].NX:=0;
   Verts[16].NY:=0;
   Verts[16].NZ:=1;

   Verts[17].X:=-2;
   Verts[17].Y:=3;
   Verts[17].Z:=4;
   Verts[17].NX:=0;
   Verts[17].NY:=0;
   Verts[17].NZ:=1;
   //
   Verts[18].X:=-2;
   Verts[18].Y:=0;
   Verts[18].Z:=-4;
   Verts[18].NX:=-1;
   Verts[18].NY:=0;
   Verts[18].NZ:=0;

   Verts[19].X:=-2;
   Verts[19].Y:=0;
   Verts[19].Z:=4;
   Verts[19].NX:=-1;
   Verts[19].NY:=0;
   Verts[19].NZ:=0;

   Verts[20].X:=-2;
   Verts[20].Y:=3;
   Verts[20].Z:=4;
   Verts[20].NX:=-1;
   Verts[20].NY:=0;
   Verts[20].NZ:=0;
   //
   Verts[21].X:=-2;
   Verts[21].Y:=0;
   Verts[21].Z:=-4;
   Verts[21].NX:=-1;
   Verts[21].NY:=0;
   Verts[21].NZ:=0;

   Verts[22].X:=-2;
   Verts[22].Y:=3;
   Verts[22].Z:=4;
   Verts[22].NX:=-1;
   Verts[22].NY:=0;
   Verts[22].NZ:=0;

   Verts[23].X:=-2;
   Verts[23].Y:=3;
   Verts[23].Z:=-4;
   Verts[23].NX:=-1;
   Verts[23].NY:=0;
   Verts[23].NZ:=0;
   //
   Verts[24].X:=-2;
   Verts[24].Y:=0;
   Verts[24].Z:=-4;
   Verts[24].NX:=0;
   Verts[24].NY:=0;
   Verts[24].NZ:=-1;

   Verts[25].X:=-2;
   Verts[25].Y:=3;
   Verts[25].Z:=-4;
   Verts[25].NX:=0;
   Verts[25].NY:=0;
   Verts[25].NZ:=-1;

   Verts[26].X:=2;
   Verts[26].Y:=3;
   Verts[26].Z:=-4;
   Verts[26].NX:=0;
   Verts[26].NY:=0;
   Verts[26].NZ:=-1;
   //
   Verts[27].X:=2;
   Verts[27].Y:=0;
   Verts[27].Z:=-4;
   Verts[27].NX:=0;
   Verts[27].NY:=0;
   Verts[27].NZ:=-1;

   Verts[28].X:=-2;
   Verts[28].Y:=0;
   Verts[28].Z:=-4;
   Verts[28].NX:=0;
   Verts[28].NY:=0;
   Verts[28].NZ:=-1;

   Verts[29].X:=2;
   Verts[29].Y:=3;
   Verts[29].Z:=-4;
   Verts[29].NX:=0;
   Verts[29].NY:=0;
   Verts[29].NZ:=-1;
   //
   Verts[30].X:=2;
   Verts[30].Y:=3;
   Verts[30].Z:=4;
   Verts[30].NX:=1;
   Verts[30].NY:=1;
   Verts[30].NZ:=0;

   Verts[31].X:=2;
   Verts[31].Y:=3;
   Verts[31].Z:=-4;
   Verts[31].NX:=1;
   Verts[31].NY:=1;
   Verts[31].NZ:=0;

   Verts[32].X:=0;
   Verts[32].Y:=5;
   Verts[32].Z:=-4;
   Verts[32].NX:=1;
   Verts[32].NY:=1;
   Verts[32].NZ:=0;
   //
   Verts[33].X:=2;
   Verts[33].Y:=3;
   Verts[33].Z:=4;
   Verts[33].NX:=1;
   Verts[33].NY:=1;
   Verts[33].NZ:=0;

   Verts[34].X:=0;
   Verts[34].Y:=5;
   Verts[34].Z:=-4;
   Verts[34].NX:=1;
   Verts[34].NY:=1;
   Verts[34].NZ:=0;

   Verts[35].X:=0;
   Verts[35].Y:=5;
   Verts[35].Z:=4;
   Verts[35].NX:=1;
   Verts[35].NY:=1;
   Verts[35].NZ:=0;
   //
   Verts[36].X:=-2;
   Verts[36].Y:=3;
   Verts[36].Z:=4;
   Verts[36].NX:=-1;
   Verts[36].NY:=1;
   Verts[36].NZ:=0;

   Verts[37].X:=0;
   Verts[37].Y:=5;
   Verts[37].Z:=-4;
   Verts[37].NX:=-1;
   Verts[37].NY:=1;
   Verts[37].NZ:=0;

   Verts[38].X:=-2;
   Verts[38].Y:=3;
   Verts[38].Z:=-4;
   Verts[38].NX:=-1;
   Verts[38].NY:=1;
   Verts[38].NZ:=0;
   //
   Verts[39].X:=0;
   Verts[39].Y:=5;
   Verts[39].Z:=4;
   Verts[39].NX:=-1;
   Verts[39].NY:=1;
   Verts[39].NZ:=0;

   Verts[40].X:=0;
   Verts[40].Y:=5;
   Verts[40].Z:=-4;
   Verts[40].NX:=-1;
   Verts[40].NY:=1;
   Verts[40].NZ:=0;

   Verts[41].X:=-2;
   Verts[41].Y:=3;
   Verts[41].Z:=4;
   Verts[41].NX:=-1;
   Verts[41].NY:=1;
   Verts[41].NZ:=0;
   //
   Verts[42].X:=2;
   Verts[42].Y:=3;
   Verts[42].Z:=4;
   Verts[42].NX:=0;
   Verts[42].NY:=0;
   Verts[42].NZ:=1;

   Verts[43].X:=0;
   Verts[43].Y:=5;
   Verts[43].Z:=4;
   Verts[43].NX:=0;
   Verts[43].NY:=0;
   Verts[43].NZ:=1;

   Verts[44].X:=-2;
   Verts[44].Y:=3;
   Verts[44].Z:=4;
   Verts[44].NX:=0;
   Verts[44].NY:=0;
   Verts[44].NZ:=1;
   //
   Verts[45].X:=-2;
   Verts[45].Y:=3;
   Verts[45].Z:=-4;
   Verts[45].NX:=0;
   Verts[45].NY:=0;
   Verts[45].NZ:=-1;

   Verts[46].X:=0;
   Verts[46].Y:=5;
   Verts[46].Z:=-4;
   Verts[46].NX:=0;
   Verts[46].NY:=0;
   Verts[46].NZ:=-1;

   Verts[47].X:=2;
   Verts[47].Y:=3;
   Verts[47].Z:=-4;
   Verts[47].NX:=0;
   Verts[47].NY:=0;
   Verts[47].NZ:=-1;

   //  
   HRet:=D3DD.CreateVertexBuffer(SizeOf(Verts),//   
                          D3DUSAGE_WRITEONLY,//    
                          D3DFVF_CUSTOMVERTEX,//  
                          D3DPOOL_DEFAULT,//     
                          VB);//  

   if FAILED(HRet) then
   begin
      CleanUp;
      MessageBox(handle,PChar(DXGErrorString (HRet)),'CreateVertexBuffer',0);
   end;
   // 
   VB.Lock(0,SizeOf(verts),PVerts,0);
   //  
   Move(Verts,PVerts^,SizeOf(Verts));
   // 
   VB.Unlock;
   //        
   D3DD.SetStreamSource(0,VB,SizeOf(TCUSTOMVERTEX));
   //  
   Result:=D3DD.SetVertexShader(D3DFVF_CUSTOMVERTEX);
end;

function TD3DFrm.RenderScene;
var
  matView,matProj:TD3DMatrix;//   
  matRotateY:TD3DMatrix;// 
begin
   //     Y
   SetRotateYMatrix(matRotateY, Angle);
   //   
   D3DD.SetTransform(D3DTS_WORLD,matRotateY);
   //  
   SetViewMatrix(matView,D3DVector(0,7,-12),D3DVector(0,0,0),D3DVector(0,1,0));
   //   
   D3DD.SetTransform(D3DTS_VIEW,matView);
   //  
   SetProjectionMatrix(matProj,1,1,1,50);
   //   
   D3DD.SetTransform(D3DTS_PROJECTION,matProj);
   // 
   D3DD.Clear(0,nil,D3DCLEAR_Target or D3DCLEAR_ZBUFFER,D3DCOLOR_XRGB(255,255,255),1,0);
   // 
   D3DD.BeginScene;
   //  
   D3DD.SetMaterial(RedM);
   //    
   D3DD.DrawPrimitive(D3DPT_TRIANGLELIST, 0,11);
   //  
   D3DD.SetMaterial(GreenM);
   //  
   D3DD.DrawPrimitive(D3DPT_TRIANGLELIST,30,6);
   // 
   D3DD.EndScene;
   //  
   Result:=D3DD.Present(nil,nil,0,nil);
end;

procedure TD3DFrm.EnableLighting;
var
   Light:TD3DLight8;//  
begin
   Light:=InitDirectionalLight(D3DVector(0, 0, 1), 1, 1, 1, 50);//   

   D3DD.SetLight(0, Light);//  

   D3DD.LightEnable(0,true);//  

   RedM:=InitMaterial(1,0,0,0);//   

   GreenM:=InitMaterial(0,1,0,0);//   
end;

procedure TD3DFrm.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
var
   HRet:HResult;
begin
   Angle:=Angle+0.01;
   HRet:=RenderScene;
   if FAILED(HRet) then
   begin
      CleanUp;
      MessageBox(handle,PChar(DXGErrorString (HRet)),'RenderScene',0);
   end;
   done:=false;
end;

end.
